use std::process::ExitCode;
use std::sync::mpsc;
use crate::mpsc::Sender;
use std::{io, process, thread, time};

mod mxnet2;
use mxnet2::License as mxnet2license;

fn setup_license(mxnet2_license: &mxnet2license, tx:Sender<i16>) -> bool
{
     if mxnet2_license.acquire() ==false {
        println!("cannot acquire license");
        return false;
    }

    mxnet2_license.retain(tx);
    true
}

fn main() -> ExitCode {


    //license is in "main" scope
    //drop is automatically called when main is out of scope
    let license =mxnet2license::new(1234, 7, 5); //usercode=1234, appslot=7, interval=5
    
	//
	//プログラムで IP/ポート設定
	// mxnetapi.ini の設定より優先
//    if license.set_ip("mxnet2b.local") == false 
//    {
//        println!("*** failed to set ip. IP will be read from mxnetapi.ini ***");
//    }
//    license.set_port(12300);

    let (tx, rx) = mpsc::channel();

    // start license check in background
    let tx1 = tx.clone();
    if setup_license(&license, tx1) == false {
        return ExitCode::FAILURE;
    }

    //CTRL+Cハンドラ
    let tx2 = tx.clone();
    ctrlc::set_handler( move || {
        println!("received Ctrl+C!");
        tx2.send(2).unwrap();
    })
    .expect("Error setting Ctrl-C handler");

    //ENTERキー 待ちスレッド
    let tx3 = tx.clone();
    thread::spawn( move || {
        let mut key_in = String::new();
        let ret = io::stdin().read_line(&mut key_in);
        match ret{
            Ok(_key) =>{ tx3.send(3).unwrap();},
            Err(_msg) => {tx3.send(3).unwrap();}
        }

    });

    //ライセンス更新エラー、CTRL+C, ENTER 発生待ちスレッド
    //発生したらプログラム終了
    let lic1 = license.clone();
     thread::spawn( move || {
         let received = rx.recv().unwrap(); 
         lic1.release();
         println!("Released");
         println!("終了します ( 原因:{} )", received);
        process::exit(0);
    });

    //プログラム本体処理 
    //ここでは２分待ちするだけ
    let wait_second = time::Duration::from_secs(120);
    thread::sleep(wait_second);

    ExitCode::SUCCESS
}
